
package fast;

import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import at.mabs.util.cuckoo.CIntHashMap;
import cern.colt.function.IntObjectProcedure;
import cern.colt.map.OpenIntObjectHashMap;

import com.google.caliper.Param;
import com.google.caliper.SimpleBenchmark;

import fast.maps.FastIntMap;
import fast.maps.IntMap;
import fast.maps.IntMap.Entries;
import gnu.trove.iterator.TIntObjectIterator;
import gnu.trove.map.hash.TIntObjectHashMap;
import gnu.trove.procedure.TIntObjectProcedure;

public class IntMapIterate extends SimpleBenchmark {
	@Param int count;

	int[] randomInt;
	final Object value = new Object();

	FastIntMap fastIntMap_Nate;
	TIntObjectHashMap tIntObjectHashMap_Trove;
	OpenIntObjectHashMap openIntObjectHashMap_Cern;
	IntMap intMap_Nate;
	CIntHashMap cIntHashMap_d3ltor;
	IntMap intMap2_Nate;

	protected void setUp () throws Exception {
		randomInt = new int[count];
		Random r = new Random();
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			randomInt[i] = value;
		}

		fastIntMap_Nate = new FastIntMap(count);
		for (int i = 0; i < count; i++)
			fastIntMap_Nate.put(randomInt[i], value);

		tIntObjectHashMap_Trove = new TIntObjectHashMap(count);
		for (int i = 0; i < count; i++)
			tIntObjectHashMap_Trove.put(randomInt[i], value);

		openIntObjectHashMap_Cern = new OpenIntObjectHashMap(count);
		for (int i = 0; i < count; i++)
			openIntObjectHashMap_Cern.put(randomInt[i], value);

		intMap_Nate = new IntMap(count);
		for (int i = 0; i < count; i++)
			intMap_Nate.put(randomInt[i], value);

		intMap2_Nate = new IntMap(count);
		for (int i = 0; i < count; i++)
			intMap2_Nate.put(randomInt[i], value);

		cIntHashMap_d3ltor = new CIntHashMap(count, 0.6f);
		for (int i = 0; i < count; i++)
			cIntHashMap_d3ltor.put(randomInt[i], value);
	}

	public int timeFastIntMap_Nate (int reps) {
		int dummy = 0;
		for (int rep = 0; rep < reps; rep++)
			for (Iterator<FastIntMap.Entry<Object>> iter = fastIntMap_Nate.iterator(); iter.hasNext();)
				dummy += iter.next().getKey();
		return dummy;
	}

	public int timeTIntObjectHashMap_Trove (int reps) {
		int dummy = 0;
		for (int rep = 0; rep < reps; rep++)
			for (TIntObjectIterator iter = tIntObjectHashMap_Trove.iterator(); iter.hasNext();) {
				iter.advance();
				dummy += iter.key();
			}
		return dummy;
	}

	public int timeTIntObjectHashMap_Trove_callback (int reps) {
		dummy = 0;
		for (int rep = 0; rep < reps; rep++)
			tIntObjectHashMap_Trove.forEachEntry(tintObjectProcedure);
		return dummy;
	}

	public int timeOpenIntObjectHashMap_Cern_callback (int reps) {
		dummy = 0;
		for (int rep = 0; rep < reps; rep++)
			openIntObjectHashMap_Cern.forEachPair(intObjectProcedure);
		return dummy;
	}

	public int timeIntMap2_Nate (int reps) {
		int dummy = 0;
		for (int rep = 0; rep < reps; rep++) {
			Entries<Object> entries = intMap_Nate.entries();
			for (IntMap.Entry<Object> e : entries)
				dummy += e.key;
		}
		return dummy;
	}

	public int timeIntMap_Nate (int reps) {
		int dummy = 0;
		for (int rep = 0; rep < reps; rep++) {
			Entries<Object> entries = intMap_Nate.entries();
			for (IntMap.Entry<Object> e : entries)
				dummy += e.key;
		}
		return dummy;
	}

	public int timeCIntHashMap_d3ltor (int reps) {
		int dummy = 0;
		for (int rep = 0; rep < reps; rep++) {
			for (Object o : cIntHashMap_d3ltor.entrySet())
				dummy += (Integer)((Map.Entry)o).getKey();
		}
		return dummy;
	}

	static public int dummy;

	static private TIntObjectProcedure tintObjectProcedure = new TIntObjectProcedure() {
		public boolean execute (int key, Object value) {
			dummy += key;
			return true;
		}
	};

	static private IntObjectProcedure intObjectProcedure = new IntObjectProcedure() {
		public boolean apply (int key, Object value) {
			dummy += key;
			return true;
		}
	};
}
