
package fast;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;

import org.apache.commons.collections.FastHashMap;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.collections.map.IdentityMap;

import at.mabs.util.cuckoo.CHashMap;

import com.google.caliper.Param;
import com.google.caliper.SimpleBenchmark;

import fast.maps.CachingFastMap;
import fast.maps.FastIdentityMap;
import fast.maps.FastMap;
import fast.maps.ObjectMap;
import fast.other.CachingHashMap;
import fast.other.HashMapV2;
import gnu.trove.map.hash.THashMap;
import gnu.trove.procedure.TObjectObjectProcedure;

public class MapIterate extends SimpleBenchmark {
	@Param int count;

	Integer[] randomInteger;
	final Object value = new Object();

	HashMap hashMap_Java;
	Hashtable hashtable_Java;
	IdentityHashMap identityHashMap_Java;
	CachingHashMap cachingHashMap_JL235;
	FastMap fastMap_Nate;
	FastIdentityMap fastIdentityMap_Nate;
	CachingFastMap cachingFastMap_Nate;
	CHashMap cHashMap_delt0r;
	THashMap tHashMap_Trove;
	HashedMap hashedMap_CommonsCollections;
	IdentityMap identityMap_CommonsCollections;
	FastHashMap fastHashMap_CommonsCollections;
	HashMapV2 hashMapV2_DougLea;
	ObjectMap objectMap_Nate;
	fast.maps.IdentityMap identityMap_Nate;
	LinkedHashMap linkedHashMap_Java;

	protected void setUp () throws Exception {
		randomInteger = new Integer[count];
		Random r = new Random();
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			randomInteger[i] = value;
		}

		hashMap_Java = new HashMap(count);
		for (int i = 0; i < count; i++)
			hashMap_Java.put(randomInteger[i], value);

		hashtable_Java = new Hashtable(count);
		for (int i = 0; i < count; i++)
			hashtable_Java.put(randomInteger[i], value);

		identityHashMap_Java = new IdentityHashMap(count);
		for (int i = 0; i < count; i++)
			identityHashMap_Java.put(randomInteger[i], value);

		linkedHashMap_Java = new LinkedHashMap(count);
		for (int i = 0; i < count; i++)
			linkedHashMap_Java.put(randomInteger[i], value);

		cachingHashMap_JL235 = new CachingHashMap(count);
		for (int i = 0; i < count; i++)
			cachingHashMap_JL235.put(randomInteger[i], value);

		fastMap_Nate = new FastMap(count);
		for (int i = 0; i < count; i++)
			fastMap_Nate.put(randomInteger[i], value);

		fastIdentityMap_Nate = new FastIdentityMap(count);
		for (int i = 0; i < count; i++)
			fastIdentityMap_Nate.put(randomInteger[i], value);

		cachingFastMap_Nate = new CachingFastMap(count);
		for (int i = 0; i < count; i++)
			cachingFastMap_Nate.put(randomInteger[i], value);

		cHashMap_delt0r = new CHashMap(count, 0.6f);
		for (int i = 0; i < count; i++)
			cHashMap_delt0r.put(randomInteger[i], value);

		tHashMap_Trove = new THashMap(count);
		for (int i = 0; i < count; i++)
			tHashMap_Trove.put(randomInteger[i], value);

		hashedMap_CommonsCollections = new HashedMap(count);
		for (int i = 0; i < count; i++)
			hashedMap_CommonsCollections.put(randomInteger[i], value);

		identityMap_CommonsCollections = new IdentityMap(count);
		for (int i = 0; i < count; i++)
			identityMap_CommonsCollections.put(randomInteger[i], value);

		fastHashMap_CommonsCollections = new FastHashMap(count);
		for (int i = 0; i < count; i++)
			fastHashMap_CommonsCollections.put(randomInteger[i], value);

		hashMapV2_DougLea = new HashMapV2(count);
		for (int i = 0; i < count; i++)
			hashMapV2_DougLea.put(randomInteger[i], value);

		objectMap_Nate = new ObjectMap(count);
		for (int i = 0; i < count; i++)
			objectMap_Nate.put(randomInteger[i], value);

		identityMap_Nate = new fast.maps.IdentityMap(count);
		for (int i = 0; i < count; i++)
			identityMap_Nate.put(randomInteger[i], value);
	}

	public void timeHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = hashMap_Java.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeHashtable_Java (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = hashtable_Java.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeIdentityHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = identityHashMap_Java.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeLinkedHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = linkedHashMap_Java.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeCachingHashMap_JL235 (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = cachingHashMap_JL235.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeFastMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = fastMap_Nate.iterator(); iter.hasNext();)
				((FastMap.Entry)iter.next()).getKey();
	}

	public void timeFastIdentityMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = fastIdentityMap_Nate.iterator(); iter.hasNext();)
				((FastIdentityMap.Entry)iter.next()).getKey();
	}

	public void timeCachingFastMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = cachingFastMap_Nate.iterator(); iter.hasNext();)
				((CachingFastMap.Entry)iter.next()).getKey();
	}

	public void timeCHashMap_delt0r (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = cHashMap_delt0r.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeTHashMap_Trove_callback (int reps) {
		for (int rep = 0; rep < reps; rep++)
			tHashMap_Trove.forEachEntry(tobjectObjectProcedure);
	}

	public void timeHashedMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = hashedMap_CommonsCollections.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeIdentityMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = identityMap_CommonsCollections.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeFastHashMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = fastHashMap_CommonsCollections.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeHashMapV2_DougLea (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = hashMapV2_DougLea.entrySet().iterator(); iter.hasNext();)
				((Map.Entry)iter.next()).getKey();
	}

	public void timeObjectMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = objectMap_Nate.entries(); iter.hasNext();)
				iter.next();
	}

	public void timeIdentityMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++)
			for (Iterator iter = identityMap_Nate.entries(); iter.hasNext();)
				iter.next();
	}

	static private TObjectObjectProcedure tobjectObjectProcedure = new TObjectObjectProcedure() {
		public boolean execute (Object key, Object value) {
			return true;
		}
	};
}
