
package fast;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.IdentityHashMap;
import java.util.LinkedHashMap;
import java.util.Random;

import org.apache.commons.collections.FastHashMap;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.collections.map.IdentityMap;

import at.mabs.util.cuckoo.CHashMap;

import com.google.caliper.Param;
import com.google.caliper.SimpleBenchmark;

import fast.maps.CachingFastMap;
import fast.maps.FastIdentityMap;
import fast.maps.FastMap;
import fast.maps.ObjectMap;
import fast.other.CachingHashMap;
import fast.other.HashMapV2;
import gnu.trove.map.hash.THashMap;

public class MapPut extends SimpleBenchmark {
	@Param int count;

	Integer[] randomInteger;
	final Object value = new Object();

	HashMap hashMap_Java;
	Hashtable hashtable_Java;
	IdentityHashMap identityHashMap_Java;
	CachingHashMap cachingHashMap_JL235;
	FastMap fastMap_Nate;
	FastIdentityMap fastIdentityMap_Nate;
	CachingFastMap cachingFastMap_Nate;
	CHashMap cHashMap_delt0r;
	THashMap tHashMap_Trove;
	HashedMap hashedMap_CommonsCollections;
	IdentityMap identityMap_CommonsCollections;
	FastHashMap fastHashMap_CommonsCollections;
	HashMapV2 hashMapV2_DougLea;
	ObjectMap objectMap_Nate;
	IdentityMap identityMap_Nate;
	LinkedHashMap linkedHashMap_Java;

	protected void setUp () throws Exception {
		randomInteger = new Integer[count];
		Random r = new Random();
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			randomInteger[i] = value;
		}

		hashMap_Java = new HashMap(count);
		hashtable_Java = new Hashtable(count);
		identityHashMap_Java = new IdentityHashMap(count);
		linkedHashMap_Java = new LinkedHashMap(count);
		cachingHashMap_JL235 = new CachingHashMap(count);
		fastMap_Nate = new FastMap(count);
		fastIdentityMap_Nate = new FastIdentityMap(count);
		cachingFastMap_Nate = new CachingFastMap(count);
		cHashMap_delt0r = new CHashMap(count, 0.6f);
		tHashMap_Trove = new THashMap(count);
		hashedMap_CommonsCollections = new HashedMap(count);
		identityMap_CommonsCollections = new IdentityMap(count);
		fastHashMap_CommonsCollections = new FastHashMap(count);
		hashMapV2_DougLea = new HashMapV2(count);
		objectMap_Nate = new ObjectMap(count);
		identityMap_Nate = new IdentityMap(count);
	}

	public void timeHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				hashMap_Java.put(randomInteger[i], value);
			hashMap_Java.clear();
		}
	}

	public void timeHashtable_Java (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				hashtable_Java.put(randomInteger[i], value);
			hashtable_Java.clear();
		}
	}

	public void timeIdentityHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				identityHashMap_Java.put(randomInteger[i], value);
			identityHashMap_Java.clear();
		}
	}

	public void timeLinkedHashMap_Java (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				linkedHashMap_Java.put(randomInteger[i], value);
			linkedHashMap_Java.clear();
		}
	}

	public void timeCachingHashMap_JL235 (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				cachingHashMap_JL235.put(randomInteger[i], value);
			cachingHashMap_JL235.clear();
		}
	}

	public void timeFastMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				fastMap_Nate.put(randomInteger[i], value);
			fastMap_Nate.clear();
		}
	}

	public void timeFastIdentityMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				fastIdentityMap_Nate.put(randomInteger[i], value);
			fastIdentityMap_Nate.clear();
		}
	}

	public void timeCachingFastMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				cachingFastMap_Nate.put(randomInteger[i], value);
			cachingFastMap_Nate.clear();
		}
	}

	public void timeCHashMap_delt0r (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				cHashMap_delt0r.put(randomInteger[i], value);
			cHashMap_delt0r.clear();
		}
	}

	public void timeTHashMap_Trove (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				tHashMap_Trove.put(randomInteger[i], value);
			tHashMap_Trove.clear();
		}
	}

	public void timeHashedMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				hashedMap_CommonsCollections.put(randomInteger[i], value);
			hashedMap_CommonsCollections.clear();
		}
	}

	public void timeIdentityMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				identityMap_CommonsCollections.put(randomInteger[i], value);
			identityMap_CommonsCollections.clear();
		}
	}

	public void timeFastHashMap_CommonsCollections (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				fastHashMap_CommonsCollections.put(randomInteger[i], value);
			fastHashMap_CommonsCollections.clear();
		}
	}

	public void timeHashMapV2_DougLea (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				hashMapV2_DougLea.put(randomInteger[i], value);
			hashMapV2_DougLea.clear();
		}
	}

	public void timeObjectMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				objectMap_Nate.put(randomInteger[i], value);
			objectMap_Nate.clear();
		}
	}

	public void timeIdentityMap_Nate (int reps) {
		for (int rep = 0; rep < reps; rep++) {
			for (int i = 0; i < count; i++)
				identityMap_Nate.put(randomInteger[i], value);
			identityMap_Nate.clear();
		}
	}
}
