
package fast;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.IdentityHashMap;
import java.util.Map.Entry;
import java.util.Random;

import org.apache.commons.collections.FastHashMap;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.collections.map.IdentityMap;

import at.mabs.util.cuckoo.CHashMap;
import cern.colt.map.OpenIntObjectHashMap;
import fast.maps.CachingFastMap;
import fast.maps.FastIdentityMap;
import fast.maps.FastIntMap;
import fast.maps.IntMap;
import fast.maps.ObjectMap;
import fast.other.ApacheIntHashMap;
import fast.other.CachingHashMap;
import fast.other.FastHashMap2;
import fast.other.HashMapV2;
import gnu.trove.map.hash.THashMap;
import gnu.trove.map.hash.TIntObjectHashMap;

public class MapTest {
	static int count = 100000;
	static String system = "Desktop";
	// static String system = "Android";

	static Integer[] randomInteger = new Integer[count];
	static int[] randomInt = new int[count];
	static boolean warmup = true;
	static HashMap<String, Long> testTimes = new HashMap();
	static ArrayList<Test> tests = new ArrayList();

	public static void main (String[] args) throws Exception {
		Random r = new Random();
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			randomInt[i] = value;
			randomInteger[i] = value;
		}

		addTest("HashMap, Java", new HashMapTest());
		addTest("Hashtable, Java", new HashtableTest());
		addTest("IdentityHashMap, Java", new IdentityHashMapTest());
		// addTest("FastMap, Javolution", new FastMapTest());
		addTest("CachingHashMap, JL235", new CachingHashMapTest());
		addTest("FastMap, Nate", new FastMap2Test());
		addTest("FastIdentityMap, Nate", new FastIdentityMapTest());
		addTest("FastIntMap, Nate", new FastIntMapTest());
		addTest("CachingFastMap, Nate", new CachingFastMapTest());
		addTest("CHashMap (cuckoo), delt0r", new CHashMapTest());
		addTest("THashMap, Trove", new THashMapTest());
		addTest("TIntObjectHashMap, Trove", new TIntObjectHashMapTest());
		addTest("OpenIntObjectHashMap, Cern Cole", new OpenIntObjectHashMapTest());
		addTest("HashedMap, Commons Collections", new HashedMapTest());
		addTest("IdentityMap, Commons Collections", new IdentityMapTestCC());
		addTest("FastHashMap, Commons Collections", new FastHashMapTest());
		addTest("ApacheIntHashMap, Commons Lang", new ApacheIntHashMapTest());
		addTest("HashMapV2, Doug Lea", new HashMapV2Test());
		addTest("IntMap, Nate", new IntMapTest());
		addTest("ObjectMap, Nate", new ObjectMapTest());
		addTest("IdentityMap, Nate", new IdentityMapTest());

		if (!system.equals("Android")) {
			addTest("FastHashMap, alex14n", new FastHashMap2Test());
			// addTest("Int2OpenObjectMap, fastutil", new Int2OpenObjectMapTest());
			// addTest("Object2OpenObjectMap, fastutil", new Object2OpenObjectMapTest());
		}

		for (Test runnable : tests) {
			warmup = true;
			for (int i = 0; i < 5; i++)
				runnable.run();
			warmup = false;
			for (int i = 0; i < 5; i++)
				runnable.run();
			System.out.println();
		}

		Comparator<Entry> comparator = new Comparator<Entry>() {
			public int compare (Entry o1, Entry o2) {
				// return ((String)o1.getKey()).compareTo((String)o2.getKey());
				return (int)((Long)o1.getValue() - (Long)o2.getValue());
			}
		};

		ArrayList<Entry> list = new ArrayList(testTimes.entrySet());
		Collections.sort(list, comparator);
		chart("Get", list, " get");
		chart("Put", list, " put");

		HashMap totalTimes = new HashMap();
		for (String name : testTimes.keySet()) {
			name = name.replace(" get", "").replace(" put", "");
			long sum = 0;
			for (String name2 : testTimes.keySet())
				if (name2.equals(name + " get") || name2.equals(name + " put")) sum += testTimes.get(name2);
			totalTimes.put(name, sum);
		}
		list = new ArrayList(totalTimes.entrySet());
		Collections.sort(list, comparator);
		chart("Total", list, "");
	}

	static void addTest (String name, Test test) {
		test.name = name;
		tests.add(test);
	}

	static void chart (String title, ArrayList<Entry> list, String suffix) {
		StringBuilder names = new StringBuilder(512);
		StringBuilder times = new StringBuilder(512);
		long max = 0;
		int count = 0;
		for (Entry<String, Long> entry : list) {
			String name = entry.getKey();
			if (!name.endsWith(suffix)) continue;
			names.insert(0, '|');
			names.insert(0, name.substring(0, name.length() - suffix.length()));
			long time = entry.getValue();
			times.append(time);
			times.append(',');
			max = Math.max(max, time);
			count++;
		}
		times.setLength(times.length() - 1);
		names.setLength(names.length() - 1);
		int height = count * 18 + 21;
		int width = Math.min(700, 300000 / height);
		System.out.println("[img]http://chart.apis.google.com/chart?chtt=" + title + ", " + system + "&" + "chs=" + width + "x"
			+ height + "&chd=t:" + times + "&chds=0," + max + "&chxl=0:|" + names + "&cht=bhg&chbh=10&chxt=y&"
			+ "chco=660000|660033|660066|660099|6600CC|6600FF|663300|663333|"
			+ "663366|663399|6633CC|6633FF|666600|666633|666666[/img]\n");
	}

	static void gc () {
		if (warmup) return;
		System.gc();
		System.gc();
		Thread.yield();
		System.gc();
		System.gc();
	}

	static abstract class Test implements Runnable {
		String name;
		long s;
		Object someValue = new Object();

		void start () {
			s = System.nanoTime();
		}

		void end (String suffix) {
			long e = System.nanoTime();
			if (warmup) return;
			long time = e - s;
			Long oldTime = testTimes.get(name + " " + suffix);
			if (oldTime == null || time < oldTime) testTimes.put(name + " " + suffix, time);
			System.out.println(name + " " + suffix + ": " + time / 1000000f + " ms");
		}
	}

	static class HashMapTest extends Test {
		HashMap map = new HashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			HashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class HashtableTest extends Test {
		Hashtable map = new Hashtable(count * 2);

		public void run () {
			Object someValue = this.someValue;
			Hashtable map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

// static class FastMapTest extends Test {
// FastMap map = new FastMap(count * 2);
//
// public void run () {
// Object someValue = this.someValue;
// FastMap map = this.map;
// map.clear();
// Integer[] randomInteger = MapTest.randomInteger;
// int count = MapTest.count;
// gc();
//
// start();
// for (int i = 0; i < count; i++)
// map.put(randomInteger[i], someValue);
// end("put");
//
// start();
// for (int i = 0; i < count; i++)
// map.get(randomInteger[i]);
// end("get");
// }
// }

	static class FastIntMapTest extends Test {
		FastIntMap map = new FastIntMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			FastIntMap map = this.map;
			map.clear();
			int[] randomInt = MapTest.randomInt;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInt[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInt[i]);
			end("get");
		}
	}

	static class CHashMapTest extends Test {
		CHashMap map = new CHashMap(count * 4, 0.7);

		public void run () {
			Object someValue = this.someValue;
			CHashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class CachingFastMapTest extends Test {
		CachingFastMap map = new CachingFastMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			CachingFastMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class CachingHashMapTest extends Test {
		CachingHashMap map = new CachingHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			CachingHashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class IdentityHashMapTest extends Test {
		IdentityHashMap map = new IdentityHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			IdentityHashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class FastIdentityMapTest extends Test {
		FastIdentityMap map = new FastIdentityMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			FastIdentityMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class FastMap2Test extends Test {
		fast.maps.FastMap map = new fast.maps.FastMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			fast.maps.FastMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class IntMapTest extends Test {
		IntMap map = new IntMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			IntMap map = this.map;
			map.clear();
			int[] randomInt = MapTest.randomInt;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInt[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInt[i]);
			end("get");
		}
	}

	static class ObjectMapTest extends Test {
		ObjectMap map = new ObjectMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			ObjectMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class IdentityMapTestCC extends Test {
		IdentityMap map = new IdentityMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			IdentityMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class THashMapTest extends Test {
		THashMap map = new THashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			THashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class TIntObjectHashMapTest extends Test {
		TIntObjectHashMap map = new TIntObjectHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			TIntObjectHashMap map = this.map;
			map.clear();
			int[] randomInt = MapTest.randomInt;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInt[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInt[i]);
			end("get");
		}
	}

	static class OpenIntObjectHashMapTest extends Test {
		OpenIntObjectHashMap map = new OpenIntObjectHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			OpenIntObjectHashMap map = this.map;
			map.clear();
			int[] randomInt = MapTest.randomInt;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInt[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInt[i]);
			end("get");
		}
	}

	static class HashedMapTest extends Test {
		HashedMap map = new HashedMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			HashedMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class IdentityMapTest extends Test {
		IdentityMap map = new IdentityMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			IdentityMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class ApacheIntHashMapTest extends Test {
		ApacheIntHashMap map = new ApacheIntHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			ApacheIntHashMap map = this.map;
			map.clear();
			int[] randomInt = MapTest.randomInt;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInt[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInt[i]);
			end("get");
		}
	}

	static class FastHashMapTest extends Test {
		FastHashMap map = new FastHashMap(count * 2);

		public void run () {
			Object someValue = this.someValue;
			FastHashMap map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

	static class FastHashMap2Test extends Test {
		FastHashMap2 map = new FastHashMap2(count * 2);

		public void run () {
			Object someValue = this.someValue;
			FastHashMap2 map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}

// static class Int2OpenObjectMapTest extends Test {
// Int2ObjectOpenHashMap map = new Int2ObjectOpenHashMap(count * 2);
//
// public void run () {
// Object someValue = this.someValue;
// Int2ObjectOpenHashMap map = this.map;
// map.clear();
// int[] randomInt = MapTest.randomInt;
// int count = MapTest.count;
// gc();
//
// start();
// for (int i = 0; i < count; i++)
// map.put(randomInt[i], someValue);
// end("put");
//
// start();
// for (int i = 0; i < count; i++)
// map.get(randomInt[i]);
// end("get");
// }
// }
//
// static class Object2OpenObjectMapTest extends Test {
// Object2ObjectOpenHashMap map = new Object2ObjectOpenHashMap(count * 2);
//
// public void run () {
// Object someValue = this.someValue;
// Object2ObjectOpenHashMap map = this.map;
// map.clear();
// Integer[] randomInteger = MapTest.randomInteger;
// int count = MapTest.count;
// gc();
//
// start();
// for (int i = 0; i < count; i++)
// map.put(randomInteger[i], someValue);
// end("put");
//
// start();
// for (int i = 0; i < count; i++)
// map.get(randomInteger[i]);
// end("get");
// }
// }

	static class HashMapV2Test extends Test {
		HashMapV2 map = new HashMapV2(count * 2);

		public void run () {
			Object someValue = this.someValue;
			HashMapV2 map = this.map;
			map.clear();
			Integer[] randomInteger = MapTest.randomInteger;
			int count = MapTest.count;
			gc();

			start();
			for (int i = 0; i < count; i++)
				map.put(randomInteger[i], someValue);
			end("put");

			start();
			for (int i = 0; i < count; i++)
				map.get(randomInteger[i]);
			end("get");
		}
	}
}
