
package fast.maps;

import java.util.Random;

public class Cuckoo2<V> {
	static private final int EMPTY = Integer.MIN_VALUE;
	private static final int PRIME1 = 0xbe1f14b1;
	private static final int PRIME2 = 0xb4b82e39;
	private static final int PRIME3 = 0xced1c241;

	public int size;

	int[] keyTable;
	V[] valueTable;

	private float loadFactor;
	private int hashShift, mask, capacity, threshold;
	private int moo;

	public Cuckoo2 (int initialCapacity, float loadFactor) {
		if (initialCapacity > 1 << 30) throw new IllegalArgumentException("initialCapacity is too large.");
		if (initialCapacity < 0) throw new IllegalArgumentException("initialCapacity must be greater than zero.");
		if (loadFactor <= 0) throw new IllegalArgumentException("initialCapacity must be greater than zero.");
		capacity = nextPowerOfTwo((int)(initialCapacity * (1 / loadFactor)));
		hashShift = 31 - Integer.numberOfTrailingZeros(capacity);
		this.loadFactor = loadFactor;
		threshold = (int)(capacity * loadFactor);
		keyTable = new int[capacity];
		for (int i = keyTable.length - 1; i >= 0; i--)
			keyTable[i] = EMPTY;
		valueTable = (V[])new Object[capacity];
		mask = capacity - 1;
	}

	public void put (int key, V value) {
		int[] keyTable = this.keyTable;
		int mask = this.mask;

		int index1 = hash1(key) & mask;
		int existingKey1 = keyTable[index1];
		if (existingKey1 == key) {
			keyTable[index1] = key;
			valueTable[index1] = value;
			return;
		}

		int index2 = hash2(key) & mask;
		int existingKey2 = keyTable[index2];
		if (existingKey2 == key) {
			keyTable[index2] = key;
			valueTable[index2] = value;
			return;
		}

		if (existingKey1 == EMPTY) {
			keyTable[index1] = key;
			valueTable[index1] = value;
			if (size++ >= threshold) resize(capacity << 1);
			return;
		}

		if (existingKey2 == EMPTY) {
			keyTable[index2] = key;
			valueTable[index2] = value;
			if (size++ >= threshold) resize(capacity << 1);
			return;
		}

		int insertIndex = index1;
		int insertKey = key;
		V insertValue = value;
		V[] valueTable = this.valueTable;
		for (int i = 0, n = capacity; i < n; i++) {
			int replacedKey = keyTable[insertIndex];
			keyTable[insertIndex] = insertKey;

			if (replacedKey == EMPTY) {
				valueTable[insertIndex] = insertValue;
				if (size++ >= threshold) resize(capacity << 1);
				moo = Math.max(moo, i);
				if (i > 30) System.out.println(i);
				return;
			}

			V replacedValue = valueTable[insertIndex];
			valueTable[insertIndex] = insertValue;

			int nextIndex = hash2(replacedKey) & mask;
			insertIndex = nextIndex != insertIndex ? nextIndex : (hash1(replacedKey) & mask);
			insertKey = replacedKey;
			insertValue = replacedValue;
		}

		resize(capacity << 1);
		put(key, value);
		return;
	}

	public V get (int key) {
		int[] keyTable = this.keyTable;

		int index = hash1(key) & mask;
		if (keyTable[index] == key) return valueTable[index];

		index = hash2(key) & mask;
		if (keyTable[index] == key) return valueTable[index];

		return null;
	}

	public static void main (String[] args) {
		Cuckoo2 m = new Cuckoo2(50000, 0.9f);
		Random r = new Random();
		for (int i = 0; i < 50000; i++)
			m.put(r.nextInt(), "moo" + i);
		// System.out.println(m);
		System.out.println(m.moo);
	}

	private void resize (int newSize) {
		mask = newSize - 1;
		capacity = newSize;
		threshold = (int)(newSize * loadFactor);
		hashShift = 31 - Integer.numberOfTrailingZeros(capacity);

		int[] oldKeyTable = keyTable;
		V[] oldValueTable = valueTable;

		keyTable = new int[newSize];
		int[] keyTable = this.keyTable;
		for (int i = newSize - 1; i >= 0; i--)
			keyTable[i] = EMPTY;
		valueTable = (V[])new Object[newSize];

		for (int i = 0, n = oldKeyTable.length; i < n; i++) {
			int key = oldKeyTable[i];
			if (key == EMPTY) continue;
			put(key, oldValueTable[i]);
		}
	}

	public void clear () {
		int[] keyTable = this.keyTable;
		V[] valueTable = this.valueTable;
		for (int i = keyTable.length; i-- > 0;) {
			keyTable[i] = EMPTY;
			valueTable[i] = null;
		}
		size = 0;
	}

	private int hash1 (int h) {
		h *= PRIME1;
		return h ^ (h >>> hashShift);
	}

	private int hash2 (int h) {
		h *= PRIME2;
		return h ^ (h >>> hashShift);
	}

	public String toString () {
		if (size == 0) return "[]";
		StringBuilder buffer = new StringBuilder(32);
		buffer.append('[');

		int[] keyTable = this.keyTable;
		int i = keyTable.length;
		while (i-- > 0) {
			int key = keyTable[i];
			if (key == EMPTY) continue;
			buffer.append(key);
			buffer.append('=');
			buffer.append(valueTable[i]);
			break;
		}
		while (i-- > 0) {
			int key = keyTable[i];
			if (key == EMPTY) continue;
			buffer.append(", ");
			buffer.append(key);
			buffer.append('=');
			buffer.append(valueTable[i]);
		}
		buffer.append(']');
		return buffer.toString();
	}

	static public int nextPowerOfTwo (int value) {
		if (value == 0) return 1;
		if ((value & value - 1) == 0) return value;
		value |= value >> 1;
		value |= value >> 2;
		value |= value >> 4;
		value |= value >> 8;
		value |= value >> 16;
		return value + 1;
	}
}
