
package fast.maps;

import java.util.HashSet;
import java.util.Random;

import junit.framework.TestCase;

public class FastMapTest extends TestCase {
	public void testIntMap () {
		int count = 10000;
		Random r = new Random();
		HashSet<Integer> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add(value);
		}
		int[] keys = new int[set.size()];
		int index = 0;
		for (Integer value : set)
			keys[index++] = value;

		IntMap<Integer> map = new IntMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		map = new IntMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add(i);
		IntMap.Entries<Integer> iterator = map.entries();
		for (IntMap.Entry<Integer> e : iterator)
			values.remove(e.value);
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size);
		map.clear();
		assertEquals(0, map.size);
		assertEquals(true, map.size == 0);
		assertEquals(null, map.get(12313));
		assertEquals(null, map.get(0));
	}

	public void testFastIntMap () {
		int count = 10000;
		Random r = new Random();
		HashSet<Integer> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add(value);
		}
		int[] keys = new int[set.size()];
		int index = 0;
		for (Integer value : set)
			keys[index++] = value;

		FastIntMap<Integer> map = new FastIntMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		map = new FastIntMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add(i);
		FastIntMap.EntryIterator iterator = map.iterator();
		for (FastIntMap.Entry<Integer> e : map)
			values.remove(e.getValue());
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size());
		map.clear();
		assertEquals(0, map.size());
		assertEquals(true, map.isEmpty());
		assertEquals(null, map.get(12313));
		assertEquals(null, map.get(0));
	}

	public void testObjectMap () {
		int count = 1;
		Random r = new Random();
		HashSet<Integer> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add(value);
		}
		Integer[] keys = new Integer[set.size()];
		int index = 0;
		for (Integer value : set)
			keys[index++] = value;

		ObjectMap<Integer, Integer> map = new ObjectMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals((Integer)i, (Integer)map.get(keys[i]));

		map = new ObjectMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add(i);
		ObjectMap.Entries<Integer, Integer> iterator = map.entries();
		for (ObjectMap.Entry<Integer, Integer> e : iterator)
			values.remove(e.value);
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size);
		map.clear();
		assertEquals(0, map.size);
		assertEquals(true, map.size == 0);
		assertEquals(null, map.get(12313));
		assertEquals(null, map.get(0));
	}

	public void testFastMap () {
		int count = 10000;
		Random r = new Random();
		HashSet<Integer> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add(value);
		}
		int[] keys = new int[set.size()];
		int index = 0;
		for (Integer value : set)
			keys[index++] = value;

		FastMap<Integer, Integer> map = new FastMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		map = new FastMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], i);
		for (int i = 0; i < keys.length; i++)
			assertEquals(i, (int)map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add(i);
		FastMap.EntryIterator iterator = map.iterator();
		for (FastMap.Entry<Integer, Integer> e : map)
			values.remove(e.getValue());
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size());
		map.clear();
		assertEquals(0, map.size());
		assertEquals(true, map.isEmpty());
		assertEquals(null, map.get(12313));
		assertEquals(null, map.get(0));
	}

	public void testFastIdentityMap () {
		int count = 10000;
		Random r = new Random();
		HashSet<String> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add("key" + value);
		}
		String[] keys = new String[set.size()];
		int index = 0;
		for (String value : set)
			keys[index++] = value;

		FastIdentityMap<String, String> map = new FastIdentityMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], "value" + i);
		for (int i = 0; i < keys.length; i++)
			assertEquals("value" + i, map.get(keys[i]));

		map = new FastIdentityMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], "value" + i);
		for (int i = 0; i < keys.length; i++)
			assertEquals("value" + i, map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add("value" + i);
		FastIdentityMap.EntryIterator iterator = map.iterator();
		for (FastIdentityMap.Entry<String, String> e : map)
			values.remove(e.getValue());
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size());
		map.clear();
		assertEquals(0, map.size());
		assertEquals(true, map.isEmpty());
		assertEquals(null, map.get("12313"));
		assertEquals(null, map.get(""));
	}

	public void testIdentityMap () {
		int count = 10000;
		Random r = new Random();
		HashSet<String> set = new HashSet(count * 2);
		for (int i = 0; i < count; i++) {
			int value = -1;
			while (value == -1)
				value = r.nextInt();
			set.add("key" + value);
		}
		String[] keys = new String[set.size()];
		int index = 0;
		for (String value : set)
			keys[index++] = value;

		IdentityMap<String, String> map = new IdentityMap();
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], "value" + i);
		for (int i = 0; i < keys.length; i++)
			assertEquals("value" + i, map.get(keys[i]));

		map = new IdentityMap(set.size() * 2);
		for (int i = 0; i < keys.length; i++)
			map.put(keys[i], "value" + i);
		for (int i = 0; i < keys.length; i++)
			assertEquals("value" + i, map.get(keys[i]));

		HashSet values = new HashSet(keys.length * 2);
		for (int i = 0; i < keys.length; i++)
			values.add("value" + i);
		IdentityMap.Entries<String, String> iterator = map.entries();
		for (IdentityMap.Entry<String, String> e : iterator)
			values.remove(e.value);
		assertEquals(0, values.size());

		assertEquals(set.size(), map.size);
		map.clear();
		assertEquals(0, map.size);
		assertEquals(true, map.size == 0);
		assertEquals(null, map.get("12313"));
		assertEquals(null, map.get(""));
	}
}
