package fast.other;

/**
 * This is a small class designed for helping to detect fail fast conditions.
 *
 * To use you increment the object any time a state change is made to the object
 * it is monitoring. When creating an iterator you get the curret fail fast
 * state with 'getFailFast()'.
 *
 * The value returned is a code for the current state of this FailFast. It can
 * be checked against this object with the isFail method to check if a failure
 * has occurred.
 *
 * It is final to encourage inlining by the compiler in order to try to minimize
 * it's overhead added to the data structures operations.
 * 
 * @author Joseph Lenton - JosephLenton@StudioFortress.com
 */
final class FailFast
{
    private int failFast;

    /**
     * Trivial constructor.
     */
    public FailFast()
    {
        failFast = 0;
    }
    
    /**
     * @return A code for the current FailFast state.
     */
    public int getFailFast()
    {
        return failFast;
    }

    /**
     * Increments this to say that a state has just occurred.
     */
    public void increment()
    {
        failFast++;
    }

    /**
     *
     * @param failFastNum
     * @return False if this FailFast is in the same state as the one given, otherwise true change has occurred.
     */
    public boolean isFail(final int failFastNum)
    {
        return failFast != failFastNum;
    }
}
